# TODO / Wish / Reminder list

Please add to this as you think of things

## Build system

### PIOASM

1. Submodule, separate repo or just part of SDK proper
   1. We should allow installed pioasm (in case we don't have native compiler)
   1. Optional flex/bison regen
   1. ✔️ Dependencies on source (PIOASM changes should cause rebuild?)
   1. Rewrite
      * Not a clone of someones repo
      * Better error messages
   1. optional usage in SDK via CMake flag 

### Validate cmake build

1. ... ✔️ on Linux
1. ... ✔️ on Mac
1. ... ✔️ on Pi
1. ... ✔️ on Windows

## Validate IDE integratation 

Note we should also doc debugging setup
1. ... CLion
    1. ✔️ cmake build
    1. debugging (currently requires .gdbinit)
1. ... CodeBlocks
1. ... Visual Studio Code
    1. ✔️ cmake build
    1. debugging

### Flexibility use cases

1. Allow use of different compilers (clang/ARM Keil/x-tool)
    - clang does not support .syntax unified! - try clang + gas? 
1. Support different c runtimes (newlib, microlib, none)
1. ✔️ Support something our current native system out of the build tree without modifying the SDK
    * pre-req we can turn off some libraries ✔️ (REPLACE_xxx e.g. REPLACE_HARDWARE_UART=1)

## CMake

1. ✔️ CMAKE var guards on each hardware target

## Hardware

1. ✔️ VRB-generated hardware headers for SPI, I2C, UART
1. Punt I2C code seemingly ripped from kernel
1. ✔️ Remove ancient junk like `fpga_flash_wr_regs` from `include/hardware/regs`
   -- ✔️ make sure everything here is relevant to rp2040, and ideally none of it handwritten
1. ✔️ Clean up block names, e.g. `dw_apb_i2c_mini` -> `i2c`

### IRQ
1. ✔️ Ability to set IRQs at runtime as well as compile time
1. Ability to share IRQs at runtime
1. Default priority to the middle?

### PIO
1. ✔️ Instruction space resource management
1. Which SM resource based management
1. Which PIO resource based management

### DMA
1. Efficient DMA sharing (perhaps nothing needed after IRQ sharing)
1. Buffer absraction (this is not scoped within DMA, but it should use it)
1. Async DMA memset/memcpy/memmove (with standard future support)

## Platform

### Blocking/Timeout/Async

Want to have decoupling event object which is logcially associated with your call, which you
can wait on with timeout, wait on with others, poll etc. We want to make it trivial to
wrap a call as blocking, or whatever.... this would be trivial in C++, but in order to avoid allocation
the call site is gonna need to pass an object in, which is still fine in a macro, however
it is gonna have to appear as a parameter still

* Promise/Future style

### CRT0
   1. Split out what should be below the c-library (e.g. __main)
   1. ✔️ Run our pre-clibrary stuff (aeabi init - done as preinit) 

### C library
What do we support (note I think we must exclude what we don't from our simple lib)
   1. open and friends? (semihosting?)
   1. fopen and friends? (semihosting?)
   1. Our minimal variant, make sure we are compliant with
      * _init/_fini
      * __main
      * __cxa_*
     
      Also, do we want to support a concept of "exit" in case ppl want to flush data or whatever
      in C++ destructors or C atexit
   1. printf - currently using vsprintf from clib - this is probably large - we could provide our own
      as an option, or just rely on picolib/whatever
      * Ok i have added git@github.com:mpaland/printf.git; need to add flags to select/deselect
      * Remove sim_printf - replace underlying _putchar
      * print a character at a time without buffer when using our printf
   1. Ability to run with no clib at all (can try this with clang) 

### AEABI wrappers
   1. ✔️ float
   1. ✔️ memcpy/memcmp
   1. ✔️ bits (popcount etc.)
   1. . double (without indirection atm)
   1. ✔️ div
   1. i64
   
   Do we also need to support `libgcc` variants as per https://gcc.gnu.org/onlinedocs/gccint/Libgcc.html#Libgcc
   where they coincide with `__aeabi_` functions.
      
### Video

   1. ✔️ rename to scanvideo
   1. ✔️ fixup `#defines`
   1. reorder ??? don't recall what this means
   1. ✔️ host-mode as proof of out of source tree stuff
   1. move to 565 for DPI

#### DBI

   1. Port and fix up
   1. Separate out controller specific drivers
   1. Mode/Vsync support
        * <SPI poll (yuk)
        * <DBI poll
        * <TEAR
        * VSYNC>
    
### Audio

   1. ✔️ port
   1. fixup `#defines`
   1. Improve connection/config
       * consistent api (choose by cmake target)
   1. ✔️ Require Mono<> Stereo<>
   1. Add volume control to sample copy/conversion
   1. Add pan control to sample copy/conversion
   1. Use shared DMA support
   1. Rationalize target device selection
   1. Add USB audio host
   1. Select I2S mono/not based on source format
       * remove PIO_I2S_STEREO_INPUT
       * remove PICO_I2S_MONO_OUTPUT
     
### Locking

1. ✔️ Add auto-init for mutexes (i.e. available from startup)7
    - need host

## Libraries

### Linking

1. Produce library (.a) variants of simple targets (should be possible to do this trivially)
1. ✔️ Symlink ELF (as per graham)
1. ✔️ `NO_FLASH` should be `PICO_NO_FLASH`
1. Maybe add a flash child binary target (i.e. no stage 2, designed to be called from a host flash app)
   probably worth this having a specifiable start address
 
### Random

1. ✔️ Add a `RP_NO_HARDWARE`
1. Add `pico_config.h` for `#defines` (should allow include of user one) 
1. Build for QEMU?
1. ✔️ Consider making autogenerated register definition headers children of struct headers, rather than siblings
1. ✔️ Build option for rebooting to usb boot when returning from `main()` or when an `assert()` fires, to make UF2 development nicer

## Raspberry Pi encroachment

1. We have tbman in there right now to support `running_on_fpga()` etc. (we would at least like
to be able to move such stuff outside of the build tree as per sim-like host builds) 

## #defines

1. Convert every damn one from boolean `#ifdef` to tri-state for `#ifdef` or `#if`

## Testing

1. CI: check out examples repo and make sure we can build at least `hello_world`

## Source compliance

1. Make sure header guards are consistent
1. `.clang-format` or whatever
1. ✔️ Use `#include "foo/bar.h"` rather than `#include <foo/bar.h>` where appropriate


