/*
 * Copyright (c) 2020 Raspberry Pi (Trading) Ltd.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

// -----------------------------------------------------
// NOTE: THIS HEADER IS ALSO INCLUDED BY ASSEMBLER SO
//       SHOULD ONLY CONSIST OF PREPROCESSOR DIRECTIVES
// -----------------------------------------------------

#ifndef _BOARDS_BRINGUP_B0_H
#define _BOARDS_BRINGUP_B0_H

#ifndef PICO_DEFAULT_UART
#define PICO_DEFAULT_UART 0
#endif

#ifndef PICO_DEFAULT_UART_TX_PIN
#define PICO_DEFAULT_UART_TX_PIN 28
#endif

#ifndef PICO_DEFAULT_UART_RX_PIN
#define PICO_DEFAULT_UART_RX_PIN 29
#endif

#ifndef PICO_DEFAULT_LED_PIN
#define PICO_DEFAULT_LED_PIN 0
#endif

#define PICO_SCANVIDEO_DPI_ALPHA_PIN 15u
#endif

#ifndef PICO_SCANVIDEO_DPI_PIXEL_RSHIFT
#define PICO_SCANVIDEO_DPI_PIXEL_RSHIFT 0u
#endif

#ifndef PICO_SCANVIDEO_DPI_PIXEL_GSHIFT
#define PICO_SCANVIDEO_DPI_PIXEL_GSHIFT 5u
#endif

#ifndef PICO_SCANVIDEO_DPI_PIXEL_BSHIFT
#define PICO_SCANVIDEO_DPI_PIXEL_BSHIFT 10u

// Audio pins. I2S BCK, LRCK are on the same pins as PWM L/R.
// - When outputting I2S, PWM sees BCK and LRCK, which should sound silent as
//   they are constant duty cycle, and above the filter cutoff
// - When outputting PWM, I2S DIN should be low, so I2S should remain silent.
#define BRINGUP_B0_I2S_DIN_PIN 27
#define BRINGUP_B0_I2S_BCK_PIN 25
#define BRINGUP_B0_I2S_LRCK_PIN 26

#define BRINGUP_B0_PWM_L_PIN 28
#define BRINGUP_B0_PWM_R_PIN 27

#define BRINGUP_B0_VGA_COLOR_BASE_PIN 0
#define BRINGUP_B0_VGA_SYNC_PIN_BASE 16

#define BRINGUP_B0_SD_CLK_PIN 23
#define BRINGUP_B0_SD_CMD_PIN 24
#define BRINGUP_B0_SD_DAT0_PIN 19

#ifndef PICO_AUDIO_PWM_L_PIN
#define PICO_AUDIO_PWM_L_PIN BRINGUP_B0_PWM_L_PIN
#endif

#ifndef PICO_AUDIO_PWM_R_PIN
#define PICO_AUDIO_PWM_R_PIN BRINGUP_B0_PWM_R_PIN
#endif

#ifndef PICO_SD_CLK_PIN
#define PICO_SD_CLK_PIN BRINGUP_B0_SD_CLK_PIN
#endif

#ifndef PICO_SD_CMD_PIN
#define PICO_SD_CMD_PIN BRINGUP_B0_SD_CMD_PIN
#endif

#ifndef PICO_SD_DAT0_PIN
#define PICO_SD_DAT0_PIN BRINGUP_B0_SD_DAT0_PIN
#endif

// Note buttons are shared with VGA colour LSBs -- if using VGA, you can float
// the pin on VSYNC assertion and sample on VSYNC deassertion
#define BRINGUP_B0_BUTTON_A_PIN 0
#define BRINGUP_B0_BUTTON_B_PIN 6
#define BRINGUP_B0_BUTTON_C_PIN 11

#define PICO_AUDIO_I2S_DATA_PIN BRINGUP_B0_I2S_DIN_PIN
#define PICO_AUDIO_I2S_CLOCK_PIN_BASE BRINGUP_B0_I2S_BCK_PIN

#define PICO_FLASH_SPI_CLKDIV 4

// This is true for the W25Q40 on first-run picoboards:
#ifndef PICO_FLASH_SIZE_BYTES
#define PICO_FLASH_SIZE_BYTES (16 * 1024 * 1024)
#endif

// Drive high to force power supply into PWM mode (lower ripple on 3V3 at light loads)
//#define PICO_SMPS_MODE_PIN 23

#ifndef PICO_FLOAT_SUPPORT_ROM_V1
#define PICO_FLOAT_SUPPORT_ROM_V1 1
#endif

#ifndef PICO_DOUBLE_SUPPORT_ROM_V1
#define PICO_DOUBLE_SUPPORT_ROM_V1 1
#endif

#endif
